@@ -19,7 +19,6 @@
 
 #include <config.h>
 
-#include <string.h>
 #include <string.h>
 #include <unistd.h>
 #include <time.h>
@@ -33,11 +32,16 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
+#ifdef HAVE_CLUTTER
+#include <clutter/clutter.h>
+#endif
+
+#include <libgames-support/games-runtime.h>
+#include <libgames-support/games-string-utils.h>
+
 #include "conf.h"
 #include "util.h"
 
-#include <libgames-support/games-files.h>
-
 #include "game.h"
 
 #include "board.h"
@@ -62,7 +66,7 @@ struct _AisleriotGame
   guint32 seed;
   guint delayed_call_timeout_id;
 
-  time_t start_time;
+  GTimer *timer;
 
   double width;
   double height;
@@ -141,8 +145,11 @@ set_game_state (AisleriotGame *game,
     game->state = state;
 
     if (state == GAME_RUNNING) {
-      /* Record start time */
-      game->start_time = time (NULL);
+      /* Reset the timer */
+      g_timer_start (game->timer);
+    } else if (state >= GAME_OVER) {
+      /* Stop the timer now so we will record the right time. See bug #514239. */
+      g_timer_stop (game->timer);
     }
 
     g_object_notify (G_OBJECT (game), "state");
@@ -210,12 +217,7 @@ update_statistics (AisleriotGame *game)
   if (game->state == GAME_WON) {
     current_stats.wins++;
 
-    if (game->paused) {
-      t = game->start_time;
-    } else {
-      t = time (NULL) - game->start_time;
-    }
-
+    t = (time_t) (g_timer_elapsed (game->timer, NULL) + 0.5);
     if (t > 0) {
       if ((current_stats.best == 0) || (t < current_stats.best)) {
 	current_stats.best = t;
@@ -239,8 +241,17 @@ clear_slots (AisleriotGame *game,
   for (i = 0; i < n_slots; ++i) {
     Slot *slot = game->slots->pdata[i];
 
-    g_byte_array_free (slot->cards, TRUE);
+#ifdef HAVE_CLUTTER
+    if (slot->slot_renderer) {
+      clutter_actor_destroy (slot->slot_renderer);
+      g_object_unref (slot->slot_renderer);
+    }
+
+    g_byte_array_free (slot->old_cards, TRUE);
+#else
     g_ptr_array_free (slot->card_images, TRUE);
+#endif /* HAVE_CLUTTER */
+    g_byte_array_free (slot->cards, TRUE);
 
     g_slice_free (Slot, slot);
   }
@@ -279,59 +290,53 @@ typedef struct {
   SCM arg2;
   SCM arg3;
   SCM retval;
+  gboolean exception;
 } CallData;
 
-#define CALL_DATA_INIT  { 0, 0, 0, 0, 0, 0 };
+#define CALL_DATA_INIT  { 0, 0, 0, 0, 0, 0, 0 }
 
-static void
-cscmi_write_exception_details (int error_fd, SCM tag, SCM throw_args)
+static char *
+cscmi_exception_get_backtrace (SCM tag, SCM throw_args)
 {
   AisleriotGame *game = app_game;
-  char *message;
   SCM port;
   SCM stack;
-  char *game_name;
   GPtrArray *slots;
   guint i, n_slots;
+  GString *message;
+  char *string;
 
-  game_name = aisleriot_game_get_name (game);
-  message = g_strdup_printf ("Variation: %s\n", game_name);
-  g_free (game_name);
-  game_name = NULL;
+  message = g_string_sized_new (1024);
 
-  write (error_fd, message, strlen (message));
-  g_free (message);
+  g_string_append_printf (message, "Variation: %s\n", aisleriot_game_get_game_file (game));
+  g_string_append_printf (message, "Seed: %u\n", game->seed);
 
-  message = g_strdup_printf ("Seed: %u\n", game->seed);
-  write (error_fd, message, strlen (message));
-  g_free (message);
+  port = scm_open_output_string ();
 
-  message = "Scheme error:\n\t";
-  write (error_fd, message, strlen (message));
-
-  port = scm_fdopen (scm_from_int (error_fd),
-                     scm_mem2string ("w", sizeof (char)));
+  g_string_append (message, "Scheme error:\n\t");
   scm_display (throw_args, port);
-  scm_fsync (port);
+  string = scm_to_locale_string (scm_get_output_string (port));
+  g_string_append (message, string);
+  free (string);
 
-  message = "\nScheme tag:\n\t";
-  write (error_fd, message, strlen (message));
+  g_string_append (message, "\nScheme tag:\n\t");
   scm_display (tag, port);
-  scm_fsync (port);
+  string = scm_to_locale_string (scm_get_output_string (port));
+  g_string_append (message, string);
+  free (string);
 
-  message = "\n\nBacktrace:\n";
-  write (error_fd, message, strlen (message));
+  g_string_append (message, "\n\nBacktrace:\n");
   stack = scm_fluid_ref (SCM_VARIABLE_REF (scm_the_last_stack_fluid_var));
   if (!SCM_FALSEP (stack)) {
     scm_display_backtrace (stack, port, SCM_UNDEFINED, SCM_UNDEFINED);
-    scm_fsync (port);
+    string = scm_to_locale_string (scm_get_output_string (port));
+    g_string_append (message, string);
+    free (string);
   } else {
-    message = "\tNo backtrace available.\n";
-    write (error_fd, message, strlen (message));
+    g_string_append (message, "\tNo backtrace available.\n");
   }
 
-  message = "\n\nDeck State:\n";
-  write (error_fd, message, strlen (message));
+  g_string_append (message, "\n\nDeck State:\n");
 
   slots = aisleriot_game_get_slots (game);
 
@@ -342,9 +347,7 @@ cscmi_write_exception_details (int error_fd, SCM tag, SCM throw_args)
       GByteArray *cards = slot->cards;
       guint n_cards;
 
-      message = g_strdup_printf ("\tSlot %d\n", slot->id);
-      write (error_fd, message, strlen (message));
-      g_free (message);
+      g_string_append_printf (message, "\tSlot %d\n", slot->id);
 
       n_cards = cards->len;
       if (n_cards > 0) {
@@ -354,61 +357,72 @@ cscmi_write_exception_details (int error_fd, SCM tag, SCM throw_args)
         for (count = 0; count < n_cards; ++count) {
           Card card = CARD (data[count]);
 
-          if (count == 0)
-            message = "\t\t";
-          else
-            message = ", ";
-          write (error_fd, message, strlen (message));
+          if (count == 0) {
+            g_string_append_c (message, '\t');
+            g_string_append_c (message, '\t');
+          } else {
+            g_string_append_c (message, ' ');
+            g_string_append_c (message, ',');
+          }
+
+          g_string_append_printf (message,
+                                  "(%d %d %s)",
+                                  CARD_GET_SUIT (card),
+                                  CARD_GET_RANK (card),
+                                  /* See c2scm_card below */
+                                  CARD_GET_FACE_DOWN (card) ? "#f" : "#t");
 
-          message = g_strdup_printf ("(%d %d %s)",
-                                     CARD_GET_SUIT (card),
-                                     CARD_GET_RANK (card),
-                                     /* See c2scm_card below */
-                                     CARD_GET_FACE_DOWN (card) ? "#f" : "#t");
-          write (error_fd, message, strlen (message));
-          g_free (message);
           count++;
           if (count == 5) {
-            message = "\n";
-            write (error_fd, message, strlen (message));
+            g_string_append_c (message, '\n');
             count = 0;
           }
         }
         if (count != 0) {
-          message = "\n";
-          write (error_fd, message, strlen (message));
+          g_string_append_c (message, '\n');
         }
       } else {
-        message = "\t\t(Empty)\n";
-        write (error_fd, message, strlen (message));
+        g_string_append (message,"\t\t(Empty)\n");
       }
     }
   } else {
-    message = "\tNo cards in deck\n";
-    write (error_fd, message, strlen (message));
+    g_string_append (message, "\tNo cards in deck\n");
   }
+
+  return g_string_free (message, FALSE);
 }
 
 /* Called when we get an exception from guile.  We launch bug-buddy with the
  * exception information:
  */
 static SCM
-cscmi_catch_handler (gpointer data G_GNUC_UNUSED,
+cscmi_catch_handler (gpointer user_data,
                      SCM tag,
                      SCM throw_args)
 {
+  CallData *data = (CallData *) user_data;
   AisleriotGame *game = app_game;
+  char *message = NULL;
   int error_fd;
   char *error_file = NULL;
   GError *error = NULL;
 
+  if (data)
+    data->exception = TRUE;
+
   if (game->had_exception)
     goto out;
 
+  message = cscmi_exception_get_backtrace (tag, throw_args);
+  if (!message) {
+    g_warning ("A scheme exception occurred, but there was no exception info\n");
+    goto out;
+  }
+
   error_fd = g_file_open_tmp ("arcrashXXXXXX", &error_file, &error);
   if (error_fd >= 0) {
-    cscmi_write_exception_details (error_fd, tag, throw_args);
     close (error_fd);
+    g_file_set_contents (error_file, message, strlen (message), NULL);
 
     /* Tell the frontend about the problem */
     g_signal_emit (game, signals[EXCEPTION], 0, error_file);
@@ -418,11 +432,13 @@ cscmi_catch_handler (gpointer data G_GNUC_UNUSED,
                error->message);
     g_error_free (error);
   }
-  
+
 out:
   /* This game is over, but don't count it in the statistics */
   set_game_state (game, GAME_LOADED);
 
+  g_free (message);
+
   return SCM_UNDEFINED;
 }
 
@@ -456,9 +472,16 @@ cscmi_call_lambda (void *user_data)
 static SCM
 cscmi_eval_string (const char *string)
 {
-  return scm_internal_stack_catch (SCM_BOOL_T,
-                                   (scm_t_catch_body) scm_c_eval_string, (void *) string,
-                                   cscmi_catch_handler, NULL);
+  CallData data = CALL_DATA_INIT;
+  SCM retval;
+
+  retval = scm_internal_stack_catch (SCM_BOOL_T,
+                                     (scm_t_catch_body) scm_c_eval_string, (void *) string,
+                                     cscmi_catch_handler, &data);
+  if (data.exception)
+    return SCM_EOL;
+
+  return retval;
 }
 
 static SCM
@@ -470,23 +493,21 @@ c2scm_card (Card card)
                                        SCM_EOL)));
 }
 
-static Card
-scm2c_card (SCM card_data)
+static void
+scm2c_card (SCM card_data,
+            Card *card)
 {
-  Card card;
   guint rank, suit, face_down;
 
-  card.value = 0;
+  card->value = 0;
 
   rank = scm_to_int (SCM_CAR (card_data));
   suit = scm_to_int (SCM_CADR (card_data));
   face_down = !(SCM_NFALSEP (SCM_CADDR (card_data)));
 
-  card.attr.rank = rank;
-  card.attr.suit = suit;
-  card.attr.face_down = face_down;
-
-  return card;
+  card->attr.rank = rank;
+  card->attr.suit = suit;
+  card->attr.face_down = face_down;
 }
 
 static SCM
@@ -521,7 +542,10 @@ cscmi_slot_set_cards (Slot *slot,
     i = n_cards;
 
     for (list_el = cards; list_el != SCM_EOL; list_el = SCM_CDR (list_el)) {
-      data[--i] = CARD_UINT (scm2c_card (SCM_CAR (list_el)));
+      Card card;
+
+      scm2c_card (SCM_CAR (list_el), &card);
+      data[--i] = CARD_UINT (card);
     }
   }
 
@@ -589,7 +613,11 @@ cscmi_add_slot (SCM slot_data)
   slot->expanded_down = expanded_down != FALSE;
   slot->expanded_right = expanded_right != FALSE;
 
+#ifdef HAVE_CLUTTER
+  slot->old_cards = g_byte_array_sized_new (SLOT_CARDS_N_PREALLOC);
+#else
   slot->card_images = g_ptr_array_sized_new (SLOT_CARDS_N_PREALLOC);
+#endif
 
   slot->needs_update = TRUE;
 
@@ -933,7 +961,10 @@ scm_execute_delayed_function (SCM callback)
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+
+  if (data.exception)
+    return FALSE;
 
   aisleriot_game_test_end_of_game (game);
 
@@ -1014,12 +1045,14 @@ update_game_dealable (AisleriotGame *game)
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return;
 
   set_game_dealable (game, SCM_NFALSEP (data.retval));
 }
 
-static void
+static gboolean
 cscmi_start_game_lambda (double *width,
                          double *height)
 {
@@ -1030,10 +1063,13 @@ cscmi_start_game_lambda (double *width,
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return FALSE;
 
   *width = scm_num2double (SCM_CAR (data.retval), 0, NULL);
   *height = scm_num2double (SCM_CADR (data.retval), 0, NULL);
+  return TRUE;
 }
 
 static gboolean
@@ -1046,7 +1082,9 @@ cscmi_game_over_lambda (void)
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return TRUE;
 
   return SCM_NFALSEP (data.retval);
 }
@@ -1061,7 +1099,9 @@ cscmi_winning_game_lambda (void)
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return FALSE;
 
   return SCM_NFALSEP (data.retval);
 }
@@ -1082,7 +1122,7 @@ cscmi_eval_installed_file (const char *filename,
     return FALSE;
   }
 
-  path = games_build_filename (GAMESDIR, filename);
+  path = games_runtime_get_file (GAMES_RUNTIME_GAME_GAMES_DIRECTORY, filename);
   if (g_file_test (path, G_FILE_TEST_EXISTS) &&
       g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
     scm_c_primitive_load (path);
@@ -1091,7 +1131,7 @@ cscmi_eval_installed_file (const char *filename,
   }
 
   g_set_error (error, AISLERIOT_GAME_ERROR, GAME_ERROR_GENERIC,
-               _("Aisleriot cannot load the file \"%s\". "
+               _("Aisleriot cannot load the file “%s”. "
                  "Please check your Aisleriot installation."),
                path);
   g_free (path);
@@ -1108,6 +1148,8 @@ aisleriot_game_init (AisleriotGame *game)
 
   game->slots = g_ptr_array_sized_new (SLOT_CARDS_N_PREALLOC);
 
+  game->timer = g_timer_new ();
+
   game->timeout = 60 * 60;
 }
 
@@ -1145,6 +1187,8 @@ aisleriot_game_finalize (GObject *object)
 
   g_free (game->game_file);
 
+  g_timer_destroy (game->timer);
+
   app_game = NULL;
 
   G_OBJECT_CLASS (aisleriot_game_parent_class)->finalize (object);
@@ -1419,8 +1463,11 @@ aisleriot_game_set_paused (AisleriotGame *game,
 
   game->paused = paused;
 
-  /* (Re)store the start time */
-  game->start_time = time (NULL) - game->start_time;
+  if (paused) {
+    g_timer_stop (game->timer);
+  } else {
+    g_timer_continue (game->timer);
+  }
 }
 
 /**
@@ -1612,7 +1659,7 @@ aisleriot_game_new_game (AisleriotGame *game,
     /* FIXMEchpe: We should NOT re-seed here, but just get another random number! */
     g_random_set_seed (game->seed);
 
-    cscmi_start_game_lambda (&game->width, &game->height);
+    cscmi_start_game_lambda (&game->width, &game->height); /* FIXME this may fail */
     scm_c_eval_string ("(start-game)");
   } while (!cscmi_game_over_lambda ());
 
@@ -1663,7 +1710,7 @@ aisleriot_game_get_game_file (AisleriotGame *game)
 char *
 aisleriot_game_get_name (AisleriotGame *game)
 {
-  return aisleriot_util_get_display_filename (game->game_file);
+  return games_filename_to_display_name (game->game_file);
 }
 
 /**
@@ -1720,9 +1767,12 @@ aisleriot_game_drag_valid (AisleriotGame *game,
   scm_gc_protect_object (data.arg2);
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
   scm_gc_unprotect_object (data.arg2);
 
+  if (data.exception)
+    return FALSE;
+
   return SCM_NFALSEP (data.retval);
 }
 
@@ -1756,9 +1806,12 @@ aisleriot_game_drop_valid (AisleriotGame *game,
   data.arg3 = scm_from_int (end_slot);
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
   scm_gc_unprotect_object (data.arg2);
 
+  if (data.exception)
+    return FALSE;
+
   return SCM_NFALSEP (data.retval);
 }
 
@@ -1788,9 +1841,12 @@ aisleriot_game_drop_cards (AisleriotGame *game,
   data.arg3 = scm_from_int (end_slot);
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
   scm_gc_unprotect_object (data.arg2);
 
+  if (data.exception)
+    return FALSE;
+
   return SCM_NFALSEP (data.retval);
 }
 
@@ -1814,7 +1870,9 @@ aisleriot_game_button_clicked_lambda (AisleriotGame *game,
   data.arg1 = scm_from_int (slot_id);
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return FALSE;
 
   return SCM_NFALSEP (data.retval);
 }
@@ -1839,75 +1897,291 @@ aisleriot_game_button_double_clicked_lambda (AisleriotGame *game,
   data.arg1 = scm_from_int (slot_id);
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return FALSE;
 
   return SCM_NFALSEP (data.retval);
 }
 
 /**
- * aisleriot_game_hint_lambda:
+ * aisleriot_game_get_hint:
  * @game:
  *
  * Gets a hint.
  *
- * Returns: a #SCM containing the hint
+ * Returns: a newly allocated string containing the hint message
  */
-SCM
-aisleriot_game_hint_lambda (AisleriotGame *game)
+char *
+aisleriot_game_get_hint (AisleriotGame *game)
 {
   CallData data = CALL_DATA_INIT;
+  SCM hint, string1, string2;
+  char *message = NULL;
+  char *str1, *str2;
 
   data.lambda = game->hint_lambda;
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return NULL;
 
-  return data.retval;
+  hint = data.retval;
+
+  if (!SCM_NFALSEP (hint)) {
+    message = g_strdup (_("This game does not have hint support yet."));
+  } else {
+    switch (scm_num2int (SCM_CAR (hint), SCM_ARG1, NULL)) {
+
+    case 0:
+      string1 = SCM_CADR (hint);
+      if (!scm_is_string (string1))
+        break;
+
+      str1 = scm_to_locale_string (string1);
+      if (!str1)
+        break;
+
+      message = g_strdup (str1);
+      free (str1);
+      break;
+
+    case 1:
+      string1 = SCM_CADR (hint);
+      string2 = SCM_CADDR (hint);
+
+      if (!scm_is_string (string1) || !scm_is_string (string2))
+        break;
+
+      str1 = scm_to_locale_string (string1);
+      if (!str1)
+        break;
+      str2 = scm_to_locale_string (string2);
+      if (!str2) {
+        free (str1);
+        break;
+      }
+
+      /* Both %s are card names */
+      message = g_strdup_printf (_("Move %s onto %s."), str1, str2);
+      free (str1);
+      free (str2);
+      break;
+
+    case 2:
+      /* NOTE! This case is exactly the same as case 1, but the strings
+        * are different: the first is a card name, the 2nd a sentence fragment.
+        * NOTE! FIXMEchpe! This is bad for i18n.
+        */
+      string1 = SCM_CADR (hint);
+      string2 = SCM_CADDR (hint);
+
+      if (!scm_is_string (string1) || !scm_is_string (string2))
+        break;
+
+      str1 = scm_to_locale_string (string1);
+      if (!str1)
+        break;
+      str2 = scm_to_locale_string (string2);
+      if (!str2) {
+        free (str1);
+        break;
+      }
+
+      /* The first %s is a card name, the 2nd %s a sentence fragment.
+        * Yes, we know this is bad for i18n.
+        */
+      message = g_strdup_printf (_("Move %s onto %s."), str1, str2);
+      free (str1);
+      free (str2);
+      break;
+
+    case 3: /* This is deprecated (due to i18n issues) do not use. */
+      g_warning ("This game uses a deprecated hint method (case 3).\n"
+                 "Please file a bug at http://bugzilla.gnome.org "
+                 "including this message and the name of the game "
+                 "you were playing, which is %s.\n",
+                 aisleriot_game_get_game_file (game));
+      break;
+
+    case 4:
+      string1 = SCM_CADR (hint);
+      if (!scm_is_string (string1))
+        break;
+
+      str1 = scm_to_locale_string (string1);
+      if (!str1)
+        break;
+
+      message = g_strdup_printf (_("You are searching for a %s."), str1);
+      free (str1);
+      break;
+
+    default:
+      message = g_strdup (_("This game is unable to provide a hint."));
+      break;
+    }
+  }
+
+  return message;
 }
 
 /**
- * aisleriot_game_get_options_lambda:
+ * aisleriot_game_option_free:
+ * @option: a #AisleriotGameOption
+ *
+ * Frees @options.
+ */
+void
+aisleriot_game_option_free (AisleriotGameOption *option)
+{
+  g_return_if_fail (option != NULL);
+
+  g_free (option->display_name);
+  g_slice_free (AisleriotGameOption, option);
+}
+
+/**
+ * aisleriot_game_get_options:
  * @game:
  *
- * Returns: a #SCM containing the game options
+ * Returns: a newly allocated list containing newly allocated
+ * #AisleriotGameOption structs. 
  */
-SCM
-aisleriot_game_get_options_lambda (AisleriotGame *game)
+GList *
+aisleriot_game_get_options (AisleriotGame *game)
 {
   CallData data = CALL_DATA_INIT;
+  SCM options_list;
+  int l, i;
+  guint32 bit = 1;
+  AisleriotGameOptionType type = AISLERIOT_GAME_OPTION_CHECK;
+  GList *options = NULL;
 
   data.lambda = game->get_options_lambda;
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return NULL;
 
-  return data.retval;
+  options_list = data.retval;
+  if (scm_is_false (scm_list_p (options_list)))
+    return NULL;
+
+  l = scm_to_int (scm_length (options_list));
+  bit = 1;
+  for (i = 0; i < l; i++) {
+    SCM entry;
+
+    /* Each entry in the options list is a list consisting of a name and
+     * a variable.
+     */
+    entry = scm_list_ref (options_list, scm_from_int (i));
+    if (!scm_is_false (scm_list_p (entry))) {
+      SCM entryname;
+      char *entrynamestr;
+      gboolean entrystate;
+      AisleriotGameOption *option;
+
+      entryname = scm_list_ref (entry, scm_from_uint (0));
+      if (!scm_is_string (entryname))
+        continue; /* Shouldn't happen */
+
+      entrynamestr = scm_to_locale_string (entryname);
+      if (!entrynamestr)
+        continue;
+
+      entrystate = SCM_NFALSEP (scm_list_ref (entry, scm_from_uint (1)));
+
+      option = g_slice_new (AisleriotGameOption);
+      option->display_name = g_strdup (entrynamestr);
+      option->type = type;
+      option->value = bit;
+      option->set = entrystate != FALSE;
+
+      options = g_list_prepend (options, option);
+
+      free (entrynamestr);
+
+      bit <<= 1;
+    } else {
+      /* If we encounter an atom, change the mode. What the atom is doesn't
+      * really matter. */
+      if (type == AISLERIOT_GAME_OPTION_CHECK) {
+        type = AISLERIOT_GAME_OPTION_RADIO;
+      } else {
+        type = AISLERIOT_GAME_OPTION_CHECK;
+      }
+    }
+  }
+
+  return g_list_reverse (options);
 }
 
 /**
  * aisleriot_game_apply_options_lambda:
  * @game:
- * @options:
+ * @changed_mask:
+ * @changed_value:
  *
- * Applies @options.
+ * Applies options.
  *
- * Returns: a #SCM with status information
+ * Returns: the new options value
  */
-SCM
-aisleriot_game_apply_options_lambda (AisleriotGame *game, SCM options)
+guint32
+aisleriot_game_change_options (AisleriotGame *game,
+                               guint32 changed_mask,
+                               guint32 changed_value)
 {
   CallData data = CALL_DATA_INIT;
+  CallData data2 = CALL_DATA_INIT;
+  SCM options_list;
+  guint32 bit, value;
+  int l, i;
 
-  data.lambda = game->apply_options_lambda;
-  data.n_args = 1;
-  data.arg1 = options;
+  data.lambda = game->get_options_lambda;
+  data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return 0;
 
-  return data.retval;
+  options_list = data.retval;
+  if (scm_is_false (scm_list_p (options_list)))
+    return 0;
+
+  value = 0;
+  bit = 1;
+  l = scm_to_int (scm_length (options_list));
+  for (i = 0; i < l; i++) {
+    SCM entry;
+
+    entry = scm_list_ref (options_list, scm_from_uint (i));
+    if (scm_is_false (scm_list_p (entry)))
+      continue;
+
+    if (changed_mask & bit)
+      scm_list_set_x (entry, scm_from_uint (1), (changed_value & bit) ? SCM_BOOL_T : SCM_BOOL_F);
+  
+    if (SCM_NFALSEP (scm_list_ref (entry, scm_from_uint (1))))
+      value |= bit;
+
+    bit <<= 1;
+  }
+
+  data2.lambda = game->apply_options_lambda;
+  data2.n_args = 1;
+  data2.arg1 = options_list;
+  scm_internal_stack_catch (SCM_BOOL_T,
+                            cscmi_call_lambda, &data2,
+                            cscmi_catch_handler, &data);
+
+  return value;
 }
 
 /**
@@ -1927,7 +2201,9 @@ aisleriot_game_timeout_lambda (AisleriotGame *game)
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
-                            cscmi_catch_handler, NULL);
+                            cscmi_catch_handler, &data);
+  if (data.exception)
+    return FALSE;
 
   return SCM_NFALSEP (data.retval);
 }
@@ -2037,7 +2313,7 @@ aisleriot_game_set_click_to_move (AisleriotGame *game,
 void
 aisleriot_game_generate_exception (AisleriotGame *game)
 {
-  cscmi_eval_string ("(how-about-a-nice-crash?)");
+  cscmi_eval_string ("(/ 1 0)");
 }
 
 /**
@@ -2051,6 +2327,9 @@ aisleriot_game_generate_exception (AisleriotGame *game)
 void
 aisleriot_game_deal_cards (AisleriotGame *game)
 {
+  /* If the game hasn't started yet, start it now */
+  aisleriot_game_start (game);
+
   aisleriot_game_record_move (game, -1, NULL, 0);
 
   cscmi_eval_string ("(do-deal-next-cards)");
@@ -2058,3 +2337,41 @@ aisleriot_game_deal_cards (AisleriotGame *game)
   aisleriot_game_end_move (game);
   aisleriot_game_test_end_of_game (game);
 }
+
+#ifdef HAVE_CLUTTER
+
+void
+aisleriot_game_get_card_offset (Slot *slot,
+                                guint card_num,
+                                gboolean old_cards,
+                                gint *xoff, gint *yoff)
+{
+  gint n_cards, exposed;
+
+  if (old_cards) {
+    n_cards = (gint) slot->old_cards->len;
+    exposed = (gint) slot->old_exposed;
+  } else {
+    n_cards = (gint) slot->cards->len;
+    exposed = (gint) slot->exposed;
+  }
+
+  if (card_num >= n_cards - exposed) {
+    gint idx = card_num + exposed - n_cards;
+    *xoff = slot->pixeldx * idx;
+    *yoff = slot->pixeldy * idx;
+  } else {
+    *xoff = 0;
+    *yoff = 0;
+  }
+}
+
+void
+aisleriot_game_reset_old_cards (Slot *slot)
+{
+  g_byte_array_set_size (slot->old_cards, 0);
+  g_byte_array_append (slot->old_cards, slot->cards->data, slot->cards->len);
+  slot->old_exposed = slot->exposed;
+}
+
+#endif /* HAVE_CLUTTER */
