@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """
 """
 
@@ -32,6 +33,9 @@ class Game:
 class NetworkFeedback:
     """Template class for feedback from a network game selector"""
     
+    def addProfile(self, profile):
+        assert(False)
+    
     def setProfile(self, profile):
         assert(False)
     
@@ -125,6 +129,9 @@ class ViewFeedback:
         'showHints' sets if move hints should be shown (boolean).
         """
         assert(False)
+        
+    def showSmooth(self, doNumbering):
+        pass
     
     def saveGame(self, path):
         """Called when the user requests the game in this view to be saved.
@@ -199,6 +206,10 @@ class ViewFeedback:
         'filename' is the file to save to (string or None to save to last filename).
         """
         assert(False)
+        
+    def undo(self):
+        """Indicates the human player wants to undo their last move"""
+        assert(False)
     
     def regsign(self):
         """Indicates the human player wants to resign"""
@@ -231,6 +242,10 @@ class ViewController:
         'move' is the move made (string).
         """
         assert(False)
+        
+    def undoMove(self):
+        """Remove the last move from this view"""
+        assert(False)
     
     def render(self):
         """Request this view is redrawn"""
@@ -309,7 +324,7 @@ class UIFeedback:
         """
         """
         assert(False)
-
+        
     def onQuit(self):
         """Called when the user quits the program"""
         assert(False)
