@@ -22,15 +22,18 @@
  * USA
  */
 
+#include <config.h>
+#include <stdlib.h>
 
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 
-#include "config.h"
-#include <gnome.h>
-
-#include <games-gridframe.h>
-#include <games-stock.h>
-#include <games-sound.h>
-#include <games-conf.h>
+#include <libgames-support/games-conf.h>
+#include <libgames-support/games-gridframe.h>
+#include <libgames-support/games-help.h>
+#include <libgames-support/games-runtime.h>
+#include <libgames-support/games-sound.h>
+#include <libgames-support/games-stock.h>
 
 #include "connect4.h"
 #include "main.h"
@@ -39,8 +42,8 @@
 #include "gfx.h"
 
 #ifdef GGZ_CLIENT
-#include <games-dlg-chat.h>
-#include <games-dlg-players.h>
+#include <libgames-support/games-dlg-chat.h>
+#include <libgames-support/games-dlg-players.h>
 #include "connectx_client.h"
 #include "ggz-network.h"
 #include <ggz-embed.h>
@@ -346,7 +349,7 @@ blink_tile (gint r, gint c, gint t, gint n)
   blink_n = n;
   blink_on = FALSE;
   anim = ANIM_BLINK;
-  timeout = gtk_timeout_add (SPEED_BLINK, (GSourceFunc) on_animate, NULL);
+  timeout = g_timeout_add (SPEED_BLINK, (GSourceFunc) on_animate, NULL);
 }
 
 
@@ -389,7 +392,7 @@ stop_anim (void)
   if (timeout == 0)
     return;
   anim = ANIM_NONE;
-  gtk_timeout_remove (timeout);
+  g_source_remove (timeout);
   timeout = 0;
 }
 
@@ -727,7 +730,7 @@ on_game_hint (GtkMenuItem * m, gpointer data)
     while (timeout)
       gtk_main_iteration ();
     anim = ANIM_HINT;
-    timeout = gtk_timeout_add (SPEED_MOVE, (GSourceFunc) on_animate, NULL);
+    timeout = g_timeout_add (SPEED_MOVE, (GSourceFunc) on_animate, NULL);
   } else {
     move_cursor (column_moveto);
   }
@@ -823,7 +826,7 @@ on_game_scores (GtkMenuItem * m, gpointer data)
   gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (scorebox)->vbox), 2);
 
   g_signal_connect (GTK_OBJECT (scorebox), "destroy",
-		    GTK_SIGNAL_FUNC (gtk_widget_destroyed), &scorebox);
+		    G_CALLBACK (gtk_widget_destroyed), &scorebox);
 
   vbox = gtk_vbox_new (FALSE, 6);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
@@ -875,7 +878,7 @@ on_game_scores (GtkMenuItem * m, gpointer data)
   gtk_misc_set_alignment (GTK_MISC (label_score[NOBODY]), 1, 0.5);
 
   g_signal_connect (GTK_DIALOG (scorebox), "response",
-		    GTK_SIGNAL_FUNC (on_dialog_close), NULL);
+		    G_CALLBACK (on_dialog_close), NULL);
 
   gtk_widget_show_all (scorebox);
 
@@ -887,7 +890,7 @@ on_game_scores (GtkMenuItem * m, gpointer data)
 static void
 on_help_about (GtkAction * action, gpointer data)
 {
-  const gchar *authors[] = { "Four-in-a-Row:",
+  const gchar *authors[] = { "Four-in-a-row:",
     "  Tim Musson <trmusson@ihug.co.nz>",
     "  David Neary <bolsh@gimp.org>",
     "",
@@ -907,10 +910,10 @@ on_help_about (GtkAction * action, gpointer data)
   const gchar *documenters[] = { "Timothy Musson",
     NULL
   };
-  gchar *license = games_get_license (_("Four-in-a-Row"));
+  gchar *license = games_get_license (_(APPNAME_LONG));
 
   gtk_show_about_dialog (GTK_WINDOW (app),
-			 "name", _("Four-in-a-Row"),
+			 "name", _(APPNAME_LONG),
 			 "version", VERSION,
 			 "copyright",
 			 "Copyright \xc2\xa9 1999-2008, Tim Musson and David Neary",
@@ -930,7 +933,7 @@ on_help_about (GtkAction * action, gpointer data)
 static void
 on_help_contents (GtkAction * action, gpointer data)
 {
-  gnome_help_display ("gnect.xml", NULL, NULL);
+  games_help_display (app, "gnect", NULL);
 }
 
 
@@ -1058,7 +1061,7 @@ blink_winner (gint n)
     anim = ANIM_BLINK;
     blink_on = FALSE;
     blink_n = n;
-    timeout = gtk_timeout_add (SPEED_BLINK, (GSourceFunc) on_animate, NULL);
+    timeout = g_timeout_add (SPEED_BLINK, (GSourceFunc) on_animate, NULL);
     while (timeout)
       gtk_main_iteration ();
   }
@@ -1068,7 +1071,7 @@ blink_winner (gint n)
     anim = ANIM_BLINK;
     blink_on = FALSE;
     blink_n = n;
-    timeout = gtk_timeout_add (SPEED_BLINK, (GSourceFunc) on_animate, NULL);
+    timeout = g_timeout_add (SPEED_BLINK, (GSourceFunc) on_animate, NULL);
     while (timeout)
       gtk_main_iteration ();
   }
@@ -1078,7 +1081,7 @@ blink_winner (gint n)
     anim = ANIM_BLINK;
     blink_on = FALSE;
     blink_n = n;
-    timeout = gtk_timeout_add (SPEED_BLINK, (GSourceFunc) on_animate, NULL);
+    timeout = g_timeout_add (SPEED_BLINK, (GSourceFunc) on_animate, NULL);
     while (timeout)
       gtk_main_iteration ();
   }
@@ -1088,7 +1091,7 @@ blink_winner (gint n)
     anim = ANIM_BLINK;
     blink_on = FALSE;
     blink_n = n;
-    timeout = gtk_timeout_add (SPEED_BLINK, (GSourceFunc) on_animate, NULL);
+    timeout = g_timeout_add (SPEED_BLINK, (GSourceFunc) on_animate, NULL);
     while (timeout)
       gtk_main_iteration ();
   }
@@ -1148,8 +1151,8 @@ void
 process_move (gint c)
 {
   if (timeout) {
-    gtk_timeout_add (SPEED_DROP,
-		     (GSourceFunc) next_move, GINT_TO_POINTER (c));
+    g_timeout_add (SPEED_DROP,
+	           (GSourceFunc) next_move, GINT_TO_POINTER (c));
     return;
 
   }
@@ -1161,8 +1164,8 @@ process_move (gint c)
   } else {
     column_moveto = c;
     anim = ANIM_MOVE;
-    timeout = gtk_timeout_add (SPEED_MOVE,
-			       (GSourceFunc) on_animate, GINT_TO_POINTER (c));
+    timeout = g_timeout_add (SPEED_MOVE,
+                             (GSourceFunc) on_animate, GINT_TO_POINTER (c));
   }
 }
 
@@ -1181,9 +1184,9 @@ process_move2 (gint c)
       row = 0;
       row_dropto = r;
       anim = ANIM_DROP;
-      timeout = gtk_timeout_add (SPEED_DROP,
-				 (GSourceFunc) on_animate,
-				 GINT_TO_POINTER (c));
+      timeout = g_timeout_add (SPEED_DROP,
+                               (GSourceFunc) on_animate,
+                               GINT_TO_POINTER (c));
     }
   } else {
     play_sound (SOUND_COLUMN_FULL);
@@ -1215,8 +1218,8 @@ process_move3 (gint c)
       c = playgame (vstr, vboard) - 1;
       if (c < 0)
 	gameover = TRUE;
-      gtk_timeout_add (SPEED_DROP,
-		       (GSourceFunc) next_move, GINT_TO_POINTER (c));
+      g_timeout_add (SPEED_DROP,
+                     (GSourceFunc) next_move, GINT_TO_POINTER (c));
     }
   }
 }
@@ -1394,7 +1397,9 @@ create_app (void)
   GtkWidget *vpaned;
   GtkUIManager *ui_manager;
 
-  app = gnome_app_new (APPNAME, _("Four-in-a-Row"));
+  app = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title (GTK_WINDOW (app), _(APPNAME_LONG));
+
   gtk_window_set_default_size (GTK_WINDOW (app), DEFAULT_WIDTH, DEFAULT_HEIGHT);
   games_conf_add_window (GTK_WINDOW (app), NULL);
 
@@ -1429,7 +1434,7 @@ create_app (void)
   gtk_box_pack_start (GTK_BOX (vbox), vpaned, TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), statusbar, FALSE, FALSE, 0);
 
-  gnome_app_set_contents (GNOME_APP (app), notebook);
+  gtk_container_add (GTK_CONTAINER (app), notebook);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, NULL);
   gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), MAIN_PAGE);
 
@@ -1480,22 +1485,25 @@ create_app (void)
 int
 main (int argc, char *argv[])
 {
-  GnomeProgram *program;
   GOptionContext *context;
+  gboolean retval;
+  GError *error = NULL;
 
-  bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-  textdomain (GETTEXT_PACKAGE);
+  if (!games_runtime_init ("gnect"))
+    return 1;
 
-  g_thread_init (NULL);
   context = g_option_context_new (NULL);
+  g_option_context_add_group (context, gtk_get_option_group (TRUE));
   games_sound_add_option_group (context);
+  retval = g_option_context_parse (context, &argc, &argv, &error);
+  g_option_context_free (context);
+  if (!retval) {
+    g_print ("%s", error->message);
+    g_error_free (error);
+    exit (1);
+  }
 
-  program = gnome_program_init (APPNAME, VERSION, LIBGNOMEUI_MODULE,
-				argc, argv,
-                                GNOME_PARAM_GOPTION_CONTEXT, context,
-                                GNOME_PARAM_APP_DATADIR, DATADIR,
-				NULL);
+  g_set_application_name (_(APPNAME_LONG));
 
   games_conf_initialise (APPNAME);
 
@@ -1505,10 +1513,8 @@ main (int argc, char *argv[])
   /* init gfx */
   if (!gfx_load_pixmaps ()) {
     games_conf_shutdown ();
-    g_object_unref (program);
     exit (1);
   }
-    
 
 #ifdef GGZ_CLIENT
   network_init ();
@@ -1523,7 +1529,7 @@ main (int argc, char *argv[])
 
   games_conf_shutdown ();
 
-  g_object_unref (program);
+  games_runtime_shutdown ();
 
   return 0;
 }
