@@ -509,6 +509,10 @@ games_conf_set_property (GObject *object,
     case PROP_GAME_NAME:
       priv->game_name = g_value_dup_string (value);
       break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
   }
 }
 
