@@ -31,6 +31,7 @@
   #include "SDL_mixer.h"
 #endif 
 
+#include "games-debug.h"
 #include "games-runtime.h"
 
 #include "games-sound.h"
@@ -78,7 +79,9 @@ games_sound_thread_run (gchar * data, gchar * user_data)
 	gchar *debug;
 
 	gst_message_parse_error (message, &err, &debug);
-	g_print (_("Error playing sound: %s\n"), err->message);
+        _games_debug_print (GAMES_DEBUG_SOUND,
+                            "Error playing sound: %s\n", err->message);
+
 	g_error_free (err);
 	g_free (debug);
 
@@ -111,7 +114,8 @@ games_sound_sdl_play (const gchar *filename)
 
   wave = Mix_LoadWAV (path);
   if (wave == NULL) {
-    g_print (_("Error playing sound %s: %s\n"), path, Mix_GetError ());
+    _games_debug_print (GAMES_DEBUG_SOUND,
+                        "Error playing sound %s: %s\n", path, Mix_GetError ());
   }
 
   Mix_PlayChannel (-1, wave, 0);
@@ -131,6 +135,8 @@ games_sound_init (void)
   g_assert (g_thread_supported ());
 
   pipeline = gst_element_factory_make ("playbin", "playbin");
+  if (pipeline == NULL)
+    return;
 
   threads = g_thread_pool_new ((GFunc) games_sound_thread_run,
 			       NULL, 10, FALSE, &err);
@@ -155,7 +161,8 @@ games_sound_init (void)
   SDL_Init (SDL_INIT_AUDIO | SDL_INIT_NOPARACHUTE);
 
   if (Mix_OpenAudio (audio_rate, audio_format, audio_channels, buf_size) < 0) {
-    g_warning ("Error calling Mix_OpenAudio\n");
+    _games_debug_print (GAMES_DEBUG_SOUND,
+                        "Error calling Mix_OpenAudio\n");
     return;
   }
 #endif /* HAVE_SDL_MIXER */
@@ -196,7 +203,8 @@ games_sound_play (const gchar * filename)
   if (!sound_init)
     games_sound_init ();
 
-  g_thread_pool_push (threads, (gchar *) filename, &err);
+  if (sound_init)
+    g_thread_pool_push (threads, (gchar *) filename, &err);
 
 #elif defined(HAVE_SDL_MIXER)
 
